# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.additional_data import AdditionalData
from ..types.idempotency_key import IdempotencyKey
from ..types.page_content import PageContent
from ..types.page_identifier import PageIdentifier
from ..types.page_setting import PageSetting
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_credentials import PayabliCredentials
from ..types.payabli_pages import PayabliPages
from ..types.receipt_content import ReceiptContent
from ..types.subdomain import Subdomain

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawHostedPaymentPagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def load_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliPages]:
        """
        Loads all of a payment page's details including `pageIdentifier` and `validationCode`. This endpoint requires an `application` API token.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliPages]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/load/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliPages,
                    parse_obj_as(
                        type_=PayabliPages,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def new_page(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """

        Creates a new payment page for a paypoint.
        Note: this operation doesn't create a new paypoint, just a payment page for an existing paypoint. Paypoints are created by the Payabli team when a boarding application is approved.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "AdditionalData": additional_data,
                "credentials": convert_and_respect_annotation_metadata(
                    object_=credentials, annotation=typing.Sequence[PayabliCredentials], direction="write"
                ),
                "lastAccess": last_access,
                "pageContent": convert_and_respect_annotation_metadata(
                    object_=page_content, annotation=PageContent, direction="write"
                ),
                "pageIdentifier": page_identifier,
                "pageSettings": convert_and_respect_annotation_metadata(
                    object_=page_settings, annotation=PageSetting, direction="write"
                ),
                "published": published,
                "receiptContent": convert_and_respect_annotation_metadata(
                    object_=receipt_content, annotation=ReceiptContent, direction="write"
                ),
                "subdomain": subdomain,
                "totalAmount": total_amount,
                "validationCode": validation_code,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def save_page(
        self,
        entry: str,
        subdomain_: str,
        *,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Updates a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain_ : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain_)}",
            method="PUT",
            json={
                "AdditionalData": additional_data,
                "credentials": convert_and_respect_annotation_metadata(
                    object_=credentials, annotation=typing.Sequence[PayabliCredentials], direction="write"
                ),
                "lastAccess": last_access,
                "pageContent": convert_and_respect_annotation_metadata(
                    object_=page_content, annotation=PageContent, direction="write"
                ),
                "pageIdentifier": page_identifier,
                "pageSettings": convert_and_respect_annotation_metadata(
                    object_=page_settings, annotation=PageSetting, direction="write"
                ),
                "published": published,
                "receiptContent": convert_and_respect_annotation_metadata(
                    object_=receipt_content, annotation=ReceiptContent, direction="write"
                ),
                "subdomain": subdomain,
                "totalAmount": total_amount,
                "validationCode": validation_code,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawHostedPaymentPagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def load_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliPages]:
        """
        Loads all of a payment page's details including `pageIdentifier` and `validationCode`. This endpoint requires an `application` API token.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliPages]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/load/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliPages,
                    parse_obj_as(
                        type_=PayabliPages,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def new_page(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """

        Creates a new payment page for a paypoint.
        Note: this operation doesn't create a new paypoint, just a payment page for an existing paypoint. Paypoints are created by the Payabli team when a boarding application is approved.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "AdditionalData": additional_data,
                "credentials": convert_and_respect_annotation_metadata(
                    object_=credentials, annotation=typing.Sequence[PayabliCredentials], direction="write"
                ),
                "lastAccess": last_access,
                "pageContent": convert_and_respect_annotation_metadata(
                    object_=page_content, annotation=PageContent, direction="write"
                ),
                "pageIdentifier": page_identifier,
                "pageSettings": convert_and_respect_annotation_metadata(
                    object_=page_settings, annotation=PageSetting, direction="write"
                ),
                "published": published,
                "receiptContent": convert_and_respect_annotation_metadata(
                    object_=receipt_content, annotation=ReceiptContent, direction="write"
                ),
                "subdomain": subdomain,
                "totalAmount": total_amount,
                "validationCode": validation_code,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def save_page(
        self,
        entry: str,
        subdomain_: str,
        *,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Updates a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain_ : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain_)}",
            method="PUT",
            json={
                "AdditionalData": additional_data,
                "credentials": convert_and_respect_annotation_metadata(
                    object_=credentials, annotation=typing.Sequence[PayabliCredentials], direction="write"
                ),
                "lastAccess": last_access,
                "pageContent": convert_and_respect_annotation_metadata(
                    object_=page_content, annotation=PageContent, direction="write"
                ),
                "pageIdentifier": page_identifier,
                "pageSettings": convert_and_respect_annotation_metadata(
                    object_=page_settings, annotation=PageSetting, direction="write"
                ),
                "published": published,
                "receiptContent": convert_and_respect_annotation_metadata(
                    object_=receipt_content, annotation=ReceiptContent, direction="write"
                ),
                "subdomain": subdomain,
                "totalAmount": total_amount,
                "validationCode": validation_code,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .ach_link_types import AchLinkTypes
from .basic_table import BasicTable


class AchSection(UniversalBaseModel):
    acceptance: typing.Optional[AchLinkTypes] = None
    fees: typing.Optional[BasicTable] = None
    price: typing.Optional[BasicTable] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .web_header_parameter import WebHeaderParameter


class NotificationRequest(UniversalBaseModel):
    notification_url: typing_extensions.Annotated[str, FieldMetadata(alias="notificationUrl")] = pydantic.Field()
    """
    Complete HTTP URL receiving the notification
    """

    web_header_parameters: typing_extensions.Annotated[
        typing.Optional[typing.List[WebHeaderParameter]], FieldMetadata(alias="webHeaderParameters")
    ] = pydantic.Field(default=None)
    """
    A dictionary of key-value pairs to be inserted in the header when the notification request is submitted
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.add_payment_method_domain_api_response import AddPaymentMethodDomainApiResponse
from ..types.domain_name import DomainName
from ..types.entity_id import EntityId
from ..types.entity_type import EntityType
from ..types.payment_method_domain_api_response import PaymentMethodDomainApiResponse
from ..types.payment_method_domain_general_response import PaymentMethodDomainGeneralResponse
from .raw_client import AsyncRawPaymentMethodDomainClient, RawPaymentMethodDomainClient
from .types.add_payment_method_domain_request_apple_pay import AddPaymentMethodDomainRequestApplePay
from .types.add_payment_method_domain_request_google_pay import AddPaymentMethodDomainRequestGooglePay
from .types.delete_payment_method_domain_response import DeletePaymentMethodDomainResponse
from .types.list_payment_method_domains_response import ListPaymentMethodDomainsResponse
from .types.update_payment_method_domain_request_wallet import UpdatePaymentMethodDomainRequestWallet

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PaymentMethodDomainClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPaymentMethodDomainClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPaymentMethodDomainClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPaymentMethodDomainClient
        """
        return self._raw_client

    def add_payment_method_domain(
        self,
        *,
        apple_pay: typing.Optional[AddPaymentMethodDomainRequestApplePay] = OMIT,
        google_pay: typing.Optional[AddPaymentMethodDomainRequestGooglePay] = OMIT,
        domain_name: typing.Optional[DomainName] = OMIT,
        entity_id: typing.Optional[EntityId] = OMIT,
        entity_type: typing.Optional[EntityType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPaymentMethodDomainApiResponse:
        """
        Add a payment method domain to an organization or paypoint.

        Parameters
        ----------
        apple_pay : typing.Optional[AddPaymentMethodDomainRequestApplePay]
            Apple Pay configuration information.

        google_pay : typing.Optional[AddPaymentMethodDomainRequestGooglePay]
            Google Pay configuration information.

        domain_name : typing.Optional[DomainName]

        entity_id : typing.Optional[EntityId]

        entity_type : typing.Optional[EntityType]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPaymentMethodDomainApiResponse
            Success

        Examples
        --------
        from payabli import payabli
        from payabli.payment_method_domain import (
            AddPaymentMethodDomainRequestApplePay,
            AddPaymentMethodDomainRequestGooglePay,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.add_payment_method_domain(
            domain_name="checkout.example.com",
            entity_id=109,
            entity_type="paypoint",
            apple_pay=AddPaymentMethodDomainRequestApplePay(
                is_enabled=True,
            ),
            google_pay=AddPaymentMethodDomainRequestGooglePay(
                is_enabled=True,
            ),
        )
        """
        _response = self._raw_client.add_payment_method_domain(
            apple_pay=apple_pay,
            google_pay=google_pay,
            domain_name=domain_name,
            entity_id=entity_id,
            entity_type=entity_type,
            request_options=request_options,
        )
        return _response.data

    def cascade_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Cascades a payment method domain to all child entities. All paypoints and suborganization under this parent will inherit this domain and its settings.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.cascade_payment_method_domain(
            domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
        )
        """
        _response = self._raw_client.cascade_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    def delete_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeletePaymentMethodDomainResponse:
        """
        Delete a payment method domain. You can't delete an inherited domain, you must delete a domain at the organization level.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeletePaymentMethodDomainResponse
            Success response for a deleted payment method domain.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.delete_payment_method_domain(
            domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
        )
        """
        _response = self._raw_client.delete_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    def get_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainApiResponse:
        """
        Get the details for a payment method domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainApiResponse
            Success response that includes a payment method domain's details.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.get_payment_method_domain(
            domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
        )
        """
        _response = self._raw_client.get_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    def list_payment_method_domains(
        self,
        *,
        entity_id: typing.Optional[int] = None,
        entity_type: typing.Optional[str] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListPaymentMethodDomainsResponse:
        """
        Get a list of payment method domains that belong to a PSP, organization, or paypoint.

        Parameters
        ----------
        entity_id : typing.Optional[int]
            Identifier for the organization or paypoint.
            - For organization, provide the organization ID - For paypoint, provide the paypoint ID

        entity_type : typing.Optional[str]
            The type of entity. Valid values:
              - organization
              - paypoint
              - psp

        from_record : typing.Optional[int]
            Number of records to skip. Defaults to `0`.

        limit_record : typing.Optional[int]
            Max number of records for query response. Defaults to `20`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListPaymentMethodDomainsResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.list_payment_method_domains(
            entity_id=1147,
            entity_type="paypoint",
        )
        """
        _response = self._raw_client.list_payment_method_domains(
            entity_id=entity_id,
            entity_type=entity_type,
            from_record=from_record,
            limit_record=limit_record,
            request_options=request_options,
        )
        return _response.data

    def update_payment_method_domain(
        self,
        domain_id: str,
        *,
        apple_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        google_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Update a payment method domain's configuration values.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        apple_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        google_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success response for configuration update.

        Examples
        --------
        from payabli import payabli
        from payabli.payment_method_domain import UpdatePaymentMethodDomainRequestWallet

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.update_payment_method_domain(
            domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
            apple_pay=UpdatePaymentMethodDomainRequestWallet(
                is_enabled=False,
            ),
            google_pay=UpdatePaymentMethodDomainRequestWallet(
                is_enabled=False,
            ),
        )
        """
        _response = self._raw_client.update_payment_method_domain(
            domain_id, apple_pay=apple_pay, google_pay=google_pay, request_options=request_options
        )
        return _response.data

    def verify_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Verify a new payment method domain. If verification is successful, Apple Pay is automatically activated for the domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.payment_method_domain.verify_payment_method_domain(
            domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
        )
        """
        _response = self._raw_client.verify_payment_method_domain(domain_id, request_options=request_options)
        return _response.data


class AsyncPaymentMethodDomainClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPaymentMethodDomainClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPaymentMethodDomainClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPaymentMethodDomainClient
        """
        return self._raw_client

    async def add_payment_method_domain(
        self,
        *,
        apple_pay: typing.Optional[AddPaymentMethodDomainRequestApplePay] = OMIT,
        google_pay: typing.Optional[AddPaymentMethodDomainRequestGooglePay] = OMIT,
        domain_name: typing.Optional[DomainName] = OMIT,
        entity_id: typing.Optional[EntityId] = OMIT,
        entity_type: typing.Optional[EntityType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddPaymentMethodDomainApiResponse:
        """
        Add a payment method domain to an organization or paypoint.

        Parameters
        ----------
        apple_pay : typing.Optional[AddPaymentMethodDomainRequestApplePay]
            Apple Pay configuration information.

        google_pay : typing.Optional[AddPaymentMethodDomainRequestGooglePay]
            Google Pay configuration information.

        domain_name : typing.Optional[DomainName]

        entity_id : typing.Optional[EntityId]

        entity_type : typing.Optional[EntityType]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddPaymentMethodDomainApiResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli
        from payabli.payment_method_domain import (
            AddPaymentMethodDomainRequestApplePay,
            AddPaymentMethodDomainRequestGooglePay,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.add_payment_method_domain(
                domain_name="checkout.example.com",
                entity_id=109,
                entity_type="paypoint",
                apple_pay=AddPaymentMethodDomainRequestApplePay(
                    is_enabled=True,
                ),
                google_pay=AddPaymentMethodDomainRequestGooglePay(
                    is_enabled=True,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_payment_method_domain(
            apple_pay=apple_pay,
            google_pay=google_pay,
            domain_name=domain_name,
            entity_id=entity_id,
            entity_type=entity_type,
            request_options=request_options,
        )
        return _response.data

    async def cascade_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Cascades a payment method domain to all child entities. All paypoints and suborganization under this parent will inherit this domain and its settings.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.cascade_payment_method_domain(
                domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cascade_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    async def delete_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeletePaymentMethodDomainResponse:
        """
        Delete a payment method domain. You can't delete an inherited domain, you must delete a domain at the organization level.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeletePaymentMethodDomainResponse
            Success response for a deleted payment method domain.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.delete_payment_method_domain(
                domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    async def get_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainApiResponse:
        """
        Get the details for a payment method domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainApiResponse
            Success response that includes a payment method domain's details.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.get_payment_method_domain(
                domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_payment_method_domain(domain_id, request_options=request_options)
        return _response.data

    async def list_payment_method_domains(
        self,
        *,
        entity_id: typing.Optional[int] = None,
        entity_type: typing.Optional[str] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListPaymentMethodDomainsResponse:
        """
        Get a list of payment method domains that belong to a PSP, organization, or paypoint.

        Parameters
        ----------
        entity_id : typing.Optional[int]
            Identifier for the organization or paypoint.
            - For organization, provide the organization ID - For paypoint, provide the paypoint ID

        entity_type : typing.Optional[str]
            The type of entity. Valid values:
              - organization
              - paypoint
              - psp

        from_record : typing.Optional[int]
            Number of records to skip. Defaults to `0`.

        limit_record : typing.Optional[int]
            Max number of records for query response. Defaults to `20`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListPaymentMethodDomainsResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.list_payment_method_domains(
                entity_id=1147,
                entity_type="paypoint",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_payment_method_domains(
            entity_id=entity_id,
            entity_type=entity_type,
            from_record=from_record,
            limit_record=limit_record,
            request_options=request_options,
        )
        return _response.data

    async def update_payment_method_domain(
        self,
        domain_id: str,
        *,
        apple_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        google_pay: typing.Optional[UpdatePaymentMethodDomainRequestWallet] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Update a payment method domain's configuration values.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        apple_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        google_pay : typing.Optional[UpdatePaymentMethodDomainRequestWallet]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success response for configuration update.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli
        from payabli.payment_method_domain import UpdatePaymentMethodDomainRequestWallet

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.update_payment_method_domain(
                domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
                apple_pay=UpdatePaymentMethodDomainRequestWallet(
                    is_enabled=False,
                ),
                google_pay=UpdatePaymentMethodDomainRequestWallet(
                    is_enabled=False,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_payment_method_domain(
            domain_id, apple_pay=apple_pay, google_pay=google_pay, request_options=request_options
        )
        return _response.data

    async def verify_payment_method_domain(
        self, domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PaymentMethodDomainGeneralResponse:
        """
        Verify a new payment method domain. If verification is successful, Apple Pay is automatically activated for the domain.

        Parameters
        ----------
        domain_id : str
            The payment method domain's ID in Payabli.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaymentMethodDomainGeneralResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.payment_method_domain.verify_payment_method_domain(
                domain_id="pmd_b8237fa45c964d8a9ef27160cd42b8c5",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify_payment_method_domain(domain_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.pay_method_ach import PayMethodAch
from ...types.pay_method_credit import PayMethodCredit
from .request_schedule_payment_method_initiator import RequestSchedulePaymentMethodInitiator

RequestSchedulePaymentMethod = typing.Union[PayMethodCredit, PayMethodAch, RequestSchedulePaymentMethodInitiator]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.dbaname import Dbaname
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.legalname import Legalname
from ...types.location_code import LocationCode
from ...types.org_parent_name import OrgParentName
from ...types.remitaddress_1 import Remitaddress1
from ...types.remitaddress_2 import Remitaddress2
from ...types.remitcity import Remitcity
from ...types.remitcountry import Remitcountry
from ...types.remitstate import Remitstate
from ...types.remitzip import Remitzip
from .v_card_get_response_associated_vendor_billing_data import VCardGetResponseAssociatedVendorBillingData
from .v_card_get_response_associated_vendor_summary import VCardGetResponseAssociatedVendorSummary
from .v_card_get_response_contact import VCardGetResponseContact


class VCardGetResponseAssociatedVendor(UniversalBaseModel):
    """
    Information about the associated vendor.
    """

    vendor_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="VendorNumber")] = (
        pydantic.Field(default=None)
    )
    """
    Unique code identifying the vendor.
    """

    name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name1")] = pydantic.Field(
        default=None
    )
    """
    The primary name associated with the vendor.
    """

    name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name2")] = pydantic.Field(
        default=None
    )
    """
    Additional name information for the vendor.
    """

    ein: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EIN")] = pydantic.Field(default=None)
    """
    Employer Identification Number of the vendor.
    """

    phone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Phone")] = pydantic.Field(
        default=None
    )
    """
    Contact phone number of the vendor.
    """

    email: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Email")] = pydantic.Field(
        default=None
    )
    """
    Contact email address of the vendor.
    """

    remit_email: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="RemitEmail")] = pydantic.Field(
        default=None
    )
    """
    Email address for remittance.
    """

    address_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address1")] = pydantic.Field(
        default=None
    )
    """
    Primary address line of the vendor.
    """

    address_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address2")] = pydantic.Field(
        default=None
    )
    """
    Secondary address line of the vendor.
    """

    city: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="City")] = pydantic.Field(default=None)
    """
    City where the vendor is located.
    """

    state: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="State")] = pydantic.Field(
        default=None
    )
    """
    State where the vendor is located.
    """

    zip: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Zip")] = pydantic.Field(default=None)
    """
    ZIP code for the vendor's location.
    """

    country: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Country")] = pydantic.Field(
        default=None
    )
    """
    Country where the vendor is located.
    """

    mcc: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Mcc")] = pydantic.Field(default=None)
    """
    Merchant Category Code for the vendor.
    """

    location_code: typing_extensions.Annotated[typing.Optional[LocationCode], FieldMetadata(alias="LocationCode")] = (
        None
    )
    contacts: typing_extensions.Annotated[
        typing.Optional[typing.List[VCardGetResponseContact]], FieldMetadata(alias="Contacts")
    ] = pydantic.Field(default=None)
    """
    Array of objects describing the vendor's contacts.
    """

    billing_data: typing_extensions.Annotated[
        typing.Optional[VCardGetResponseAssociatedVendorBillingData], FieldMetadata(alias="BillingData")
    ] = pydantic.Field(default=None)
    """
    Billing data for the vendor.
    """

    payment_method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentMethod")] = (
        pydantic.Field(default=None)
    )
    """
    Preferred payment method for vendor.
    """

    vendor_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="VendorStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Status of the vendor.
    """

    vendor_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="VendorId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier for the vendor.
    """

    enrollment_status: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EnrollmentStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Enrollment status of the vendor.
    """

    summary: typing_extensions.Annotated[
        typing.Optional[VCardGetResponseAssociatedVendorSummary], FieldMetadata(alias="Summary")
    ] = pydantic.Field(default=None)
    """
    Summary of vendor's billing and transaction status.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    Legal name of the paypoint.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    DBA name of the paypoint.
    """

    paypoint_entryname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field(default=None)
    )
    """
    Entryname of the paypoint.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    parent_org_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ParentOrgId")] = (
        pydantic.Field(default=None)
    )
    """
    ID of the parent organization.
    """

    created_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="CreatedDate")] = (
        pydantic.Field(default=None)
    )
    """
    Date when the vendor record was created.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="LastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Date when the vendor's information was last updated.
    """

    remit_address_1: typing_extensions.Annotated[
        typing.Optional[Remitaddress1], FieldMetadata(alias="remitAddress1")
    ] = None
    remit_address_2: typing_extensions.Annotated[
        typing.Optional[Remitaddress2], FieldMetadata(alias="remitAddress2")
    ] = None
    remit_city: typing_extensions.Annotated[typing.Optional[Remitcity], FieldMetadata(alias="remitCity")] = None
    remit_state: typing_extensions.Annotated[typing.Optional[Remitstate], FieldMetadata(alias="remitState")] = None
    remit_zip: typing_extensions.Annotated[typing.Optional[Remitzip], FieldMetadata(alias="remitZip")] = None
    remit_country: typing_extensions.Annotated[typing.Optional[Remitcountry], FieldMetadata(alias="remitCountry")] = (
        None
    )
    payee_name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="payeeName1")] = pydantic.Field(
        default=None
    )
    """
    Primary name of the payee.
    """

    payee_name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="payeeName2")] = pydantic.Field(
        default=None
    )
    """
    Secondary name of the payee.
    """

    custom_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField1")] = (
        pydantic.Field(default=None)
    )
    """
    A custom field for additional data.
    """

    custom_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField2")] = (
        pydantic.Field(default=None)
    )
    """
    Another custom field for extra data.
    """

    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = pydantic.Field(default=None)
    """
    Account number of paypoint in the vendor side.
    """

    internal_reference_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="InternalReferenceId")
    ] = pydantic.Field(default=None)
    """
    Internal reference ID used within the system.
    """

    additional_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="additionalData")] = (
        pydantic.Field(default=None)
    )
    """
    Field for additional data, if any.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    stored_methods: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="StoredMethods")] = (
        pydantic.Field(default=None)
    )
    """
    Stored payment methods for the vendor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .last_modified import LastModified
from .optin_status import OptinStatus


class CustomerQueryRecordsCustomerConsentSms(UniversalBaseModel):
    """
    Describes the customer's SMS communications consent status.
    """

    status: typing.Optional[OptinStatus] = None
    updated_at: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .authcode import Authcode
from .order_id import OrderId


class QueryResponseData(UniversalBaseModel):
    """
    The transaction's response data.
    """

    authcode: typing.Optional[Authcode] = None
    avsresponse: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text code describing the result for address validation (applies only for card transactions).
    """

    avsresponse_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text code describing the result for address validation (applies only for card transactions).
    """

    cvvresponse: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text code describing the result for CVV validation (applies only for card transactions).
    """

    cvvresponse_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text code describing the result for CVV validation (applies only for card transactions).
    """

    emv_auth_response_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    EMV authorization response data, applicable for card transactions.
    """

    orderid: typing.Optional[OrderId] = None
    response: typing.Optional[str] = pydantic.Field(default=None)
    """
    Response text for operation: 'Success' or 'Declined'.
    """

    response_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Internal result code processing the transaction. Value 1 indicates successful operation, values 2 and 3 indicate errors.
    """

    response_code_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text describing the result. If resultCode = 1, will return 'Approved' or a general success message. If resultCode = 2 or 3, will contain the cause of the decline.
    """

    responsetext: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text describing the result. If resultCode = 1, will return 'Approved' or a general success message. If resultCode = 2 or 3, will contain the cause of the decline.
    """

    transactionid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transaction identifier in Payabli.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of transaction or operation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class GoogleWalletData(UniversalBaseModel):
    """
    The wallet data.
    """

    gateway_merchant_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="gatewayMerchantId")] = (
        pydantic.Field(default=None)
    )
    """
    The Google Pay merchant identifier.
    """

    gateway_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="gatewayId")] = pydantic.Field(
        default=None
    )
    """
    The Google Pay gateway identifier.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class OcrBillItem(UniversalBaseModel):
    item_total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTotalAmount")] = (
        None
    )
    item_tax_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTaxAmount")] = None
    item_tax_rate: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTaxRate")] = None
    item_product_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="itemProductCode")] = None
    item_product_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="itemProductName")] = None
    item_description: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="itemDescription")] = None
    item_commodity_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="itemCommodityCode")] = (
        None
    )
    item_unit_of_measure: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="itemUnitOfMeasure")
    ] = None
    item_cost: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemCost")] = None
    item_qty: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="itemQty")] = None
    item_mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="itemMode")] = None
    item_categories: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="itemCategories")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .source import Source


class GeneralEvents(UniversalBaseModel):
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Event description.
    """

    event_time: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="eventTime")] = (
        pydantic.Field(default=None)
    )
    """
    Event timestamp, in UTC.
    """

    extra_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="extraData")
    ] = pydantic.Field(default=None)
    """
    Extra data.
    """

    ref_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="refData")] = pydantic.Field(
        default=None
    )
    """
    Reference data.
    """

    source: typing.Optional[Source] = pydantic.Field(default=None)
    """
    The event source.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .method_reference_id import MethodReferenceId
from .result_code import ResultCode
from .resulttext import Resulttext


class PayabliApiResponsePaymethodDeleteResponseData(UniversalBaseModel):
    reference_id: typing_extensions.Annotated[
        typing.Optional[MethodReferenceId], FieldMetadata(alias="referenceId")
    ] = pydantic.Field(default=None)
    """
    The method's reference ID.
    """

    result_code: typing_extensions.Annotated[typing.Optional[ResultCode], FieldMetadata(alias="resultCode")] = None
    result_text: typing_extensions.Annotated[typing.Optional[Resulttext], FieldMetadata(alias="resultText")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.batch_number import BatchNumber
from ...types.bill_data import BillData
from ...types.billing_fee_detail import BillingFeeDetail
from ...types.chargeback_id import ChargebackId
from ...types.datetime_nullable import DatetimeNullable
from ...types.dbaname import Dbaname
from ...types.device_id import DeviceId
from ...types.entrypage_id import EntrypageId
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.external_processor_information import ExternalProcessorInformation
from ...types.fee_amount import FeeAmount
from ...types.ip_address import IpAddress
from ...types.legalname import Legalname
from ...types.netamountnullable import Netamountnullable
from ...types.operation import Operation
from ...types.order_id import OrderId
from ...types.org_parent_name import OrgParentName
from ...types.orgid import Orgid
from ...types.payor_id import PayorId
from ...types.paypoint_id import PaypointId
from ...types.pending_fee_amount import PendingFeeAmount
from ...types.query_c_fee_transaction import QueryCFeeTransaction
from ...types.query_payment_data import QueryPaymentData
from ...types.query_response_data import QueryResponseData
from ...types.query_transaction_events import QueryTransactionEvents
from ...types.query_transaction_payor_data import QueryTransactionPayorData
from ...types.refund_id import RefundId
from ...types.retrieval_id import RetrievalId
from ...types.returned_id import ReturnedId
from ...types.risk_action import RiskAction
from ...types.risk_action_code import RiskActionCode
from ...types.risk_flagged import RiskFlagged
from ...types.risk_flagged_on import RiskFlaggedOn
from ...types.risk_reason import RiskReason
from ...types.risk_status import RiskStatus
from ...types.source import Source
from ...types.split_funding import SplitFunding


class TransferDetailRecord(UniversalBaseModel):
    transfer_detail_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="transferDetailId")] = (
        pydantic.Field(default=None)
    )
    """
    Unique identifier for the transfer detail record
    """

    transfer_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="transferId")] = pydantic.Field(
        default=None
    )
    """
    The ID of the transfer this detail belongs to
    """

    transaction_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="transactionId")] = (
        pydantic.Field(default=None)
    )
    """
    The transaction ID in Payabli's system
    """

    transaction_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="transactionNumber")] = (
        pydantic.Field(default=None)
    )
    """
    External transaction reference number
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transaction type (credit or debit)
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    A field used to categorize the transaction details. Values include: auth, decline, refund, adj, cb, split
    """

    gross_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="grossAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The gross amount of the transaction
    """

    charge_back_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="chargeBackAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Chargeback amount deducted from transaction
    """

    returned_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="returnedAmount")] = (
        pydantic.Field(default=None)
    )
    """
    ACH return amount deducted from transaction
    """

    refund_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="refundAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Refund amount deducted from transaction
    """

    hold_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="holdAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Amount being held for fraud or risk concerns
    """

    released_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="releasedAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Previously held funds that have been released after a risk review
    """

    billing_fees_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="billingFeesAmount")
    ] = pydantic.Field(default=None)
    """
    Charges applied for transactions and services
    """

    third_party_paid_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="thirdPartyPaidAmount")
    ] = pydantic.Field(default=None)
    """
    Payments captured in the batch cycle that are deposited separately. For example,  checks or cash payments recorded in the batch but not deposited via Payabli,  or card brands making a direct transfer in certain situations.
    """

    adjustments_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="adjustmentsAmount")
    ] = pydantic.Field(default=None)
    """
    Corrections applied to Billing & Fees charges
    """

    net_transfer_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="netTransferAmount")
    ] = pydantic.Field(default=None)
    """
    The net amount after all deductions
    """

    split_funding_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="splitFundingAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount directed to split funding destinations
    """

    billing_fees_details: typing_extensions.Annotated[
        typing.Optional[typing.List[BillingFeeDetail]], FieldMetadata(alias="billingFeesDetails")
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        None
    )
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = None
    paypoint_entryname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's entryname
    """

    payment_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentTransId")] = (
        pydantic.Field(default=None)
    )
    """
    The transaction ID for the payment
    """

    connector_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ConnectorName")] = (
        pydantic.Field(default=None)
    )
    """
    The payment connector used to process the transaction
    """

    external_processor_information: typing_extensions.Annotated[
        typing.Optional[ExternalProcessorInformation], FieldMetadata(alias="ExternalProcessorInformation")
    ] = None
    gateway_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="GatewayTransId")] = (
        pydantic.Field(default=None)
    )
    """
    Internal identifier used for processing
    """

    order_id: typing_extensions.Annotated[typing.Optional[OrderId], FieldMetadata(alias="OrderId")] = None
    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    Payment method used: card, ach, or wallet
    """

    batch_number: typing_extensions.Annotated[typing.Optional[BatchNumber], FieldMetadata(alias="BatchNumber")] = None
    batch_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The amount of the batch
    """

    payor_id: typing_extensions.Annotated[typing.Optional[PayorId], FieldMetadata(alias="PayorId")] = pydantic.Field(
        default=None
    )
    """
    Unique ID for customer linked to the transaction
    """

    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        None
    )
    trans_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TransStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Status of transaction. See [the
    docs](/developers/references/money-in-statuses#money-in-transaction-status) for a
    full reference.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[PaypointId], FieldMetadata(alias="PaypointId")] = None
    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Transaction total amount (including service fee or sub-charge)
    """

    net_amount: typing_extensions.Annotated[typing.Optional[Netamountnullable], FieldMetadata(alias="NetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Net amount paid
    """

    fee_amount: typing_extensions.Annotated[typing.Optional[FeeAmount], FieldMetadata(alias="FeeAmount")] = None
    settlement_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="SettlementStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Settlement status for transaction. See [the docs](/developers/references/money-in-statuses#payment-funding-status) for a full reference.
    """

    operation: typing_extensions.Annotated[typing.Optional[Operation], FieldMetadata(alias="Operation")] = None
    response_data: typing_extensions.Annotated[
        typing.Optional[QueryResponseData], FieldMetadata(alias="ResponseData")
    ] = None
    source: typing_extensions.Annotated[typing.Optional[Source], FieldMetadata(alias="Source")] = None
    schedule_reference: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ScheduleReference")] = (
        pydantic.Field(default=None)
    )
    """
    Reference to the subscription or schedule that originated the transaction
    """

    org_id: typing_extensions.Annotated[typing.Optional[Orgid], FieldMetadata(alias="OrgId")] = None
    refund_id: typing_extensions.Annotated[typing.Optional[RefundId], FieldMetadata(alias="RefundId")] = None
    returned_id: typing_extensions.Annotated[typing.Optional[ReturnedId], FieldMetadata(alias="ReturnedId")] = None
    chargeback_id: typing_extensions.Annotated[typing.Optional[ChargebackId], FieldMetadata(alias="ChargebackId")] = (
        None
    )
    retrieval_id: typing_extensions.Annotated[typing.Optional[RetrievalId], FieldMetadata(alias="RetrievalId")] = None
    trans_additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Optional[typing.Any]], FieldMetadata(alias="TransAdditionalData")
    ] = pydantic.Field(default=None)
    """
    Additional transaction data
    """

    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="invoiceData")] = (
        pydantic.Field(default=None)
    )
    """
    Associated invoice data
    """

    entrypage_id: typing_extensions.Annotated[typing.Optional[EntrypageId], FieldMetadata(alias="EntrypageId")] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    is_validated_ach: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="IsValidatedACH")] = (
        pydantic.Field(default=None)
    )
    """
    Indicates whether the ACH account has been validated
    """

    transaction_time: typing_extensions.Annotated[
        typing.Optional[DatetimeNullable], FieldMetadata(alias="TransactionTime")
    ] = pydantic.Field(default=None)
    """
    Transaction date and time, in UTC
    """

    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    split_funding_instructions: typing_extensions.Annotated[
        typing.Optional[SplitFunding], FieldMetadata(alias="splitFundingInstructions")
    ] = None
    cfee_transactions: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryCFeeTransaction]], FieldMetadata(alias="CfeeTransactions")
    ] = None
    transaction_events: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryTransactionEvents]], FieldMetadata(alias="TransactionEvents")
    ] = None
    pending_fee_amount: typing_extensions.Annotated[
        typing.Optional[PendingFeeAmount], FieldMetadata(alias="PendingFeeAmount")
    ] = None
    risk_flagged: typing_extensions.Annotated[typing.Optional[RiskFlagged], FieldMetadata(alias="RiskFlagged")] = None
    risk_flagged_on: typing_extensions.Annotated[
        typing.Optional[RiskFlaggedOn], FieldMetadata(alias="RiskFlaggedOn")
    ] = None
    risk_status: typing_extensions.Annotated[typing.Optional[RiskStatus], FieldMetadata(alias="RiskStatus")] = None
    risk_reason: typing_extensions.Annotated[typing.Optional[RiskReason], FieldMetadata(alias="RiskReason")] = None
    risk_action: typing_extensions.Annotated[typing.Optional[RiskAction], FieldMetadata(alias="RiskAction")] = None
    risk_action_code: typing_extensions.Annotated[
        typing.Optional[RiskActionCode], FieldMetadata(alias="RiskActionCode")
    ] = None
    device_id: typing_extensions.Annotated[typing.Optional[DeviceId], FieldMetadata(alias="DeviceId")] = None
    ach_sec_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="AchSecCode")] = None
    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[AchHolderType], FieldMetadata(alias="AchHolderType")
    ] = None
    ip_address: typing_extensions.Annotated[typing.Optional[IpAddress], FieldMetadata(alias="IpAddress")] = None
    is_same_day_ach: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="IsSameDayACH")] = (
        pydantic.Field(default=None)
    )
    """
    Indicates if this was a same-day ACH transaction.
    """

    wallet_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="WalletType")] = pydantic.Field(
        default=None
    )
    """
    Type of wallet used for the transaction (if applicable)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...types.payabli_api_response_generic_2_part import PayabliApiResponseGeneric2Part
from .get_pay_link_from_id_response_response_data import GetPayLinkFromIdResponseResponseData


class GetPayLinkFromIdResponse(PayabliApiResponseGeneric2Part):
    response_data: typing_extensions.Annotated[
        typing.Optional[GetPayLinkFromIdResponseResponseData], FieldMetadata(alias="responseData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .convert_token import ConvertToken
from .tokenize_ach import TokenizeAch
from .tokenize_card import TokenizeCard

RequestTokenStoragePaymentMethod = typing.Union[TokenizeCard, TokenizeAch, ConvertToken]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .sub_footer import SubFooter
from .sub_header import SubHeader
from .template_additional_data_section import TemplateAdditionalDataSection
from .template_element import TemplateElement
from .visible import Visible


class OwnersSection(UniversalBaseModel):
    """
    Information about a business owner.
    """

    contact_email: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="contactEmail")
    ] = None
    contact_name: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="contactName")] = (
        None
    )
    contact_phone: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="contactPhone")
    ] = None
    contact_title: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="contactTitle")
    ] = None
    multiple_contacts: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="multipleContacts")] = (
        pydantic.Field(default=None)
    )
    """
    Offer add more contacts
    """

    multiple_owners: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="multipleOwners")] = (
        pydantic.Field(default=None)
    )
    """
    offer add more owners
    """

    oaddress: typing.Optional[TemplateElement] = None
    ocity: typing.Optional[TemplateElement] = None
    ocountry: typing.Optional[TemplateElement] = None
    odriverstate: typing.Optional[TemplateElement] = None
    ostate: typing.Optional[TemplateElement] = None
    ownerdob: typing.Optional[TemplateElement] = None
    ownerdriver: typing.Optional[TemplateElement] = None
    owneremail: typing.Optional[TemplateElement] = None
    ownername: typing.Optional[TemplateElement] = None
    ownerpercent: typing.Optional[TemplateElement] = None
    ownerphone_1: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="ownerphone1")] = (
        None
    )
    ownerphone_2: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="ownerphone2")] = (
        None
    )
    ownerssn: typing.Optional[TemplateElement] = None
    ownertitle: typing.Optional[TemplateElement] = None
    ozip: typing.Optional[TemplateElement] = None
    sub_footer: typing_extensions.Annotated[typing.Optional[SubFooter], FieldMetadata(alias="subFooter")] = None
    sub_header: typing_extensions.Annotated[typing.Optional[SubHeader], FieldMetadata(alias="subHeader")] = None
    visible: typing.Optional[Visible] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[TemplateAdditionalDataSection], FieldMetadata(alias="additionalData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .link_data import LinkData


class BDetails(UniversalBaseModel):
    btype: typing.Optional[LinkData] = None
    dbaname: typing.Optional[LinkData] = None
    ein: typing.Optional[LinkData] = None
    faxnumber: typing.Optional[LinkData] = None
    legalname: typing.Optional[LinkData] = None
    license: typing.Optional[LinkData] = None
    licstate: typing.Optional[LinkData] = None
    phonenumber: typing.Optional[LinkData] = None
    startdate: typing.Optional[LinkData] = None
    taxfillname: typing.Optional[LinkData] = None
    website: typing.Optional[LinkData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .auth_response import AuthResponse
    from .auth_response_response_data import AuthResponseResponseData
    from .capture_payment_details import CapturePaymentDetails
    from .capture_request import CaptureRequest
    from .capture_response import CaptureResponse
    from .capture_response_data import CaptureResponseData
    from .get_paid_response_data import GetPaidResponseData
    from .invalid_trans_status_error_type import InvalidTransStatusErrorType
    from .payabli_api_response_get_paid import PayabliApiResponseGetPaid
    from .receipt_response import ReceiptResponse
    from .refund_response import RefundResponse
    from .refund_with_instructions_response import RefundWithInstructionsResponse
    from .request_credit_payment_method import RequestCreditPaymentMethod
    from .request_payment_authorize_payment_method import RequestPaymentAuthorizePaymentMethod
    from .request_payment_validate_payment_method import RequestPaymentValidatePaymentMethod
    from .request_payment_validate_payment_method_method import RequestPaymentValidatePaymentMethodMethod
    from .response_data_refunds import ResponseDataRefunds
    from .reverse_response import ReverseResponse
    from .trans_request_body import TransRequestBody
    from .validate_response import ValidateResponse
    from .validate_response_data import ValidateResponseData
    from .void_response import VoidResponse
    from .void_response_data import VoidResponseData
_dynamic_imports: typing.Dict[str, str] = {
    "AuthResponse": ".auth_response",
    "AuthResponseResponseData": ".auth_response_response_data",
    "CapturePaymentDetails": ".capture_payment_details",
    "CaptureRequest": ".capture_request",
    "CaptureResponse": ".capture_response",
    "CaptureResponseData": ".capture_response_data",
    "GetPaidResponseData": ".get_paid_response_data",
    "InvalidTransStatusErrorType": ".invalid_trans_status_error_type",
    "PayabliApiResponseGetPaid": ".payabli_api_response_get_paid",
    "ReceiptResponse": ".receipt_response",
    "RefundResponse": ".refund_response",
    "RefundWithInstructionsResponse": ".refund_with_instructions_response",
    "RequestCreditPaymentMethod": ".request_credit_payment_method",
    "RequestPaymentAuthorizePaymentMethod": ".request_payment_authorize_payment_method",
    "RequestPaymentValidatePaymentMethod": ".request_payment_validate_payment_method",
    "RequestPaymentValidatePaymentMethodMethod": ".request_payment_validate_payment_method_method",
    "ResponseDataRefunds": ".response_data_refunds",
    "ReverseResponse": ".reverse_response",
    "TransRequestBody": ".trans_request_body",
    "ValidateResponse": ".validate_response",
    "ValidateResponseData": ".validate_response_data",
    "VoidResponse": ".void_response",
    "VoidResponseData": ".void_response_data",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AuthResponse",
    "AuthResponseResponseData",
    "CapturePaymentDetails",
    "CaptureRequest",
    "CaptureResponse",
    "CaptureResponseData",
    "GetPaidResponseData",
    "InvalidTransStatusErrorType",
    "PayabliApiResponseGetPaid",
    "ReceiptResponse",
    "RefundResponse",
    "RefundWithInstructionsResponse",
    "RequestCreditPaymentMethod",
    "RequestPaymentAuthorizePaymentMethod",
    "RequestPaymentValidatePaymentMethod",
    "RequestPaymentValidatePaymentMethodMethod",
    "ResponseDataRefunds",
    "ReverseResponse",
    "TransRequestBody",
    "ValidateResponse",
    "ValidateResponseData",
    "VoidResponse",
    "VoidResponseData",
]



# This file was auto-generated by Fern from our API Definition.

import typing

NotificationStandardRequestContentEventType = typing.Union[
    typing.Literal[
        "ApprovedPayment",
        "AuthorizedPayment",
        "DeclinedPayment",
        "OriginatedPayment",
        "SettledPayment",
        "SubscriptionCreated",
        "SubscriptionUpdated",
        "SubscriptionCanceled",
        "SubscriptionCompleted",
        "FundedPayment",
        "VoidedPayment",
        "RefundedPayment",
        "HoldTransaction",
        "ReleasedTransaction",
        "HoldBatch",
        "ReleasedBatch",
        "TransferAdjusted",
        "TransferDisabledCreditFund",
        "TransferDisabledDebitFund",
        "TransferNotAvailableBalance",
        "TransferReadyforRetry",
        "TransferResolved",
        "TransferReturn",
        "TransferSuccess",
        "TransferSuspended",
        "TransferError",
        "SendReceipt",
        "RecoveredTransaction",
        "CreatedApplication",
        "ApprovedApplication",
        "FailedBoardingApplication",
        "SubmittedApplication",
        "ActivatedMerchant",
        "ReceivedChargeBack",
        "ChargebackUpdated",
        "ReceivedRetrieval",
        "RetrievalUpdated",
        "ReceivedAchReturn",
        "HoldingApplication",
        "DeclinedApplication",
        "BoardingApplication",
        "PaypointMoved",
        "FraudAlert",
        "InvoiceSent",
        "InvoicePaid",
        "InvoiceCreated",
        "BillPaid",
        "BillApproved",
        "BillDisApproved",
        "BillCanceled",
        "BillProcessing",
        "CardCreated",
        "CardActivated",
        "CardDeactivated",
        "CardExpired",
        "CardExpiring",
        "CardLimitUpdated",
        "BatchClosed",
        "BatchNotClosed",
        "PayOutFunded",
        "PayOutProcessed",
        "PayOutCanceled",
        "PayOutPaid",
        "PayOutReturned",
        "PayoutSubscriptionCreated",
        "PayoutSubscriptionUpdated",
        "PayoutSubscriptionCanceled",
        "PayoutSubscriptionCompleted",
        "PayoutSubscriptionReminder",
        "importFileReceived",
        "importFileProcessed",
        "importFileError",
        "exportFileSent",
        "exportFileError",
        "UpdatedMerchant",
        "Report",
        "FailedEmailNotification",
        "FailedWebNotification",
        "FailedSMSNotification",
        "UserPasswordExpiring",
        "UserPasswordExpired",
        "TransactionNotFound",
        "SystemAlert",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .account_number import AccountNumber
from .routing_account import RoutingAccount


class TransferBankAccount(UniversalBaseModel):
    account_number: typing_extensions.Annotated[AccountNumber, FieldMetadata(alias="accountNumber")]
    routing_number: typing_extensions.Annotated[RoutingAccount, FieldMetadata(alias="routingNumber")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .batch_number import BatchNumber
from .bill_data import BillData
from .category import Category
from .created_at import CreatedAt
from .deposit_date import DepositDate
from .expected_deposit_date import ExpectedDepositDate
from .external_paypoint_id import ExternalPaypointId
from .maskedaccount import Maskedaccount
from .netamountnullable import Netamountnullable
from .order_id import OrderId
from .org_parent_name import OrgParentName
from .query_payment_data import QueryPaymentData
from .query_response_data import QueryResponseData
from .query_transaction_events import QueryTransactionEvents
from .query_transaction_payor_data import QueryTransactionPayorData
from .settlement_status import SettlementStatus
from .source import Source
from .trans_status import TransStatus
from .transaction_time import TransactionTime


class QueryResponseSettlementsRecordsItem(UniversalBaseModel):
    batch_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The batch amount.
    """

    batch_number: typing_extensions.Annotated[typing.Optional[BatchNumber], FieldMetadata(alias="BatchNumber")] = None
    category: typing_extensions.Annotated[typing.Optional[Category], FieldMetadata(alias="Category")] = None
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt")] = None
    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    deposit_date: typing_extensions.Annotated[typing.Optional[DepositDate], FieldMetadata(alias="DepositDate")] = None
    expected_deposit_date: typing_extensions.Annotated[
        typing.Optional[ExpectedDepositDate], FieldMetadata(alias="ExpectedDepositDate")
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    gateway_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="GatewayTransId")] = (
        pydantic.Field(default=None)
    )
    """
    Internal identifier used for processing.
    """

    id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Id")] = None
    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="invoiceData")] = None
    is_hold: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="isHold")] = pydantic.Field(
        default=None
    )
    """
    
    Describes whether the transaction is being held or not.
    
    1 - Transaction is held
    
    0 - Transaction isn't being held
    """

    masked_account: typing_extensions.Annotated[
        typing.Optional[Maskedaccount], FieldMetadata(alias="MaskedAccount")
    ] = None
    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    The payment method.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[Netamountnullable], FieldMetadata(alias="NetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Net amount paid.
    """

    operation: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Operation")] = pydantic.Field(
        default=None
    )
    """
    The operation performed.
    """

    order_id: typing_extensions.Annotated[typing.Optional[OrderId], FieldMetadata(alias="OrderId")] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        None
    )
    payment_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentTransId")] = (
        pydantic.Field(default=None)
    )
    """
    The transaction ID for the payment.
    """

    payment_trans_status: typing_extensions.Annotated[
        typing.Optional[TransStatus], FieldMetadata(alias="PaymentTransStatus")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint entryname.
    """

    paypoint_legalname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointLegalname")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint legal name.
    """

    response_data: typing_extensions.Annotated[
        typing.Optional[QueryResponseData], FieldMetadata(alias="ResponseData")
    ] = None
    schedule_reference: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ScheduleReference")] = (
        pydantic.Field(default=None)
    )
    """
    Reference to the subscription originating the transaction.
    """

    settled_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="SettledAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The transaction amount.
    """

    settlement_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="SettlementDate")
    ] = pydantic.Field(default=None)
    """
     
    """

    source: typing_extensions.Annotated[typing.Optional[Source], FieldMetadata(alias="Source")] = None
    status: typing_extensions.Annotated[typing.Optional[SettlementStatus], FieldMetadata(alias="Status")] = None
    transaction_events: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryTransactionEvents]], FieldMetadata(alias="TransactionEvents")
    ] = pydantic.Field(default=None)
    """
    Events associated with this transaction.
    """

    transaction_time: typing_extensions.Annotated[
        typing.Optional[TransactionTime], FieldMetadata(alias="TransactionTime")
    ] = None
    trans_method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="TransMethod")] = (
        pydantic.Field(default=None)
    )
    """
    Payment method used: card or ach.
    """

    type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Type")] = pydantic.Field(default=None)
    """
    The transaction type: credit or debit.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.bill_data import BillData
from ...types.entrypointfield import Entrypointfield
from ...types.payment_detail import PaymentDetail
from ...types.payor_data_request import PayorDataRequest
from ...types.schedule_detail import ScheduleDetail
from ...types.source import Source
from ...types.subdomain import Subdomain
from .request_schedule_payment_method import RequestSchedulePaymentMethod
from .set_pause import SetPause


class SubscriptionRequestBody(UniversalBaseModel):
    customer_data: typing_extensions.Annotated[
        typing.Optional[PayorDataRequest], FieldMetadata(alias="customerData")
    ] = pydantic.Field(default=None)
    """
    Object describing the customer/payor.
    """

    entry_point: typing_extensions.Annotated[typing.Optional[Entrypointfield], FieldMetadata(alias="entryPoint")] = None
    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="invoiceData")] = (
        pydantic.Field(default=None)
    )
    """
    Object describing an Invoice linked to the subscription.
    """

    payment_details: typing_extensions.Annotated[
        typing.Optional[PaymentDetail], FieldMetadata(alias="paymentDetails")
    ] = pydantic.Field(default=None)
    """
    Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.
    """

    payment_method: typing_extensions.Annotated[
        typing.Optional[RequestSchedulePaymentMethod], FieldMetadata(alias="paymentMethod")
    ] = pydantic.Field(default=None)
    """
    Information about the payment method for the transaction. Required and recommended fields for each payment method type are described in each schema below.
    """

    schedule_details: typing_extensions.Annotated[
        typing.Optional[ScheduleDetail], FieldMetadata(alias="scheduleDetails")
    ] = pydantic.Field(default=None)
    """
    Object describing the schedule for subscription.
    """

    set_pause: typing_extensions.Annotated[typing.Optional[SetPause], FieldMetadata(alias="setPause")] = None
    source: typing.Optional[Source] = None
    subdomain: typing.Optional[Subdomain] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum


class payabliEnvironment(enum.Enum):
    SANDBOX = "https://api-sandbox.payabli.com/api"
    PRODUCTION = "https://api.payabli.com/api"



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .apple_wallet_data import AppleWalletData
from .entry import Entry
from .is_enabled import IsEnabled


class ApplePayPaypointRegistrationData(UniversalBaseModel):
    entry: typing.Optional[Entry] = None
    is_enabled: typing_extensions.Annotated[typing.Optional[IsEnabled], FieldMetadata(alias="isEnabled")] = None
    wallet_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="walletType")] = pydantic.Field(
        default=None
    )
    """
    The wallet type. In this context it will always be `applePay`.
    """

    wallet_data: typing_extensions.Annotated[typing.Optional[AppleWalletData], FieldMetadata(alias="walletData")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import payabliEnvironment

if typing.TYPE_CHECKING:
    from .bill.client import AsyncBillClient, BillClient
    from .boarding.client import AsyncBoardingClient, BoardingClient
    from .charge_backs.client import AsyncChargeBacksClient, ChargeBacksClient
    from .check_capture.client import AsyncCheckCaptureClient, CheckCaptureClient
    from .cloud.client import AsyncCloudClient, CloudClient
    from .customer.client import AsyncCustomerClient, CustomerClient
    from .export.client import AsyncExportClient, ExportClient
    from .hosted_payment_pages.client import AsyncHostedPaymentPagesClient, HostedPaymentPagesClient
    from .import_.client import AsyncImportClient, ImportClient
    from .invoice.client import AsyncInvoiceClient, InvoiceClient
    from .line_item.client import AsyncLineItemClient, LineItemClient
    from .money_in.client import AsyncMoneyInClient, MoneyInClient
    from .money_out.client import AsyncMoneyOutClient, MoneyOutClient
    from .notification.client import AsyncNotificationClient, NotificationClient
    from .notificationlogs.client import AsyncNotificationlogsClient, NotificationlogsClient
    from .ocr.client import AsyncOcrClient, OcrClient
    from .organization.client import AsyncOrganizationClient, OrganizationClient
    from .payment_link.client import AsyncPaymentLinkClient, PaymentLinkClient
    from .payment_method_domain.client import AsyncPaymentMethodDomainClient, PaymentMethodDomainClient
    from .paypoint.client import AsyncPaypointClient, PaypointClient
    from .query.client import AsyncQueryClient, QueryClient
    from .statistic.client import AsyncStatisticClient, StatisticClient
    from .subscription.client import AsyncSubscriptionClient, SubscriptionClient
    from .templates.client import AsyncTemplatesClient, TemplatesClient
    from .token_storage.client import AsyncTokenStorageClient, TokenStorageClient
    from .user.client import AsyncUserClient, UserClient
    from .vendor.client import AsyncVendorClient, VendorClient
    from .wallet.client import AsyncWalletClient, WalletClient


class payabli:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : payabliEnvironment
        The environment to use for requests from the client. from .environment import payabliEnvironment



        Defaults to payabliEnvironment.SANDBOX



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from payabli import payabli

    client = payabli(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: payabliEnvironment = payabliEnvironment.SANDBOX,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._bill: typing.Optional[BillClient] = None
        self._boarding: typing.Optional[BoardingClient] = None
        self._charge_backs: typing.Optional[ChargeBacksClient] = None
        self._check_capture: typing.Optional[CheckCaptureClient] = None
        self._cloud: typing.Optional[CloudClient] = None
        self._customer: typing.Optional[CustomerClient] = None
        self._export: typing.Optional[ExportClient] = None
        self._hosted_payment_pages: typing.Optional[HostedPaymentPagesClient] = None
        self._import_: typing.Optional[ImportClient] = None
        self._invoice: typing.Optional[InvoiceClient] = None
        self._line_item: typing.Optional[LineItemClient] = None
        self._money_in: typing.Optional[MoneyInClient] = None
        self._money_out: typing.Optional[MoneyOutClient] = None
        self._notification: typing.Optional[NotificationClient] = None
        self._notificationlogs: typing.Optional[NotificationlogsClient] = None
        self._ocr: typing.Optional[OcrClient] = None
        self._organization: typing.Optional[OrganizationClient] = None
        self._payment_link: typing.Optional[PaymentLinkClient] = None
        self._payment_method_domain: typing.Optional[PaymentMethodDomainClient] = None
        self._paypoint: typing.Optional[PaypointClient] = None
        self._query: typing.Optional[QueryClient] = None
        self._statistic: typing.Optional[StatisticClient] = None
        self._subscription: typing.Optional[SubscriptionClient] = None
        self._templates: typing.Optional[TemplatesClient] = None
        self._token_storage: typing.Optional[TokenStorageClient] = None
        self._user: typing.Optional[UserClient] = None
        self._vendor: typing.Optional[VendorClient] = None
        self._wallet: typing.Optional[WalletClient] = None

    @property
    def bill(self):
        if self._bill is None:
            from .bill.client import BillClient  # noqa: E402

            self._bill = BillClient(client_wrapper=self._client_wrapper)
        return self._bill

    @property
    def boarding(self):
        if self._boarding is None:
            from .boarding.client import BoardingClient  # noqa: E402

            self._boarding = BoardingClient(client_wrapper=self._client_wrapper)
        return self._boarding

    @property
    def charge_backs(self):
        if self._charge_backs is None:
            from .charge_backs.client import ChargeBacksClient  # noqa: E402

            self._charge_backs = ChargeBacksClient(client_wrapper=self._client_wrapper)
        return self._charge_backs

    @property
    def check_capture(self):
        if self._check_capture is None:
            from .check_capture.client import CheckCaptureClient  # noqa: E402

            self._check_capture = CheckCaptureClient(client_wrapper=self._client_wrapper)
        return self._check_capture

    @property
    def cloud(self):
        if self._cloud is None:
            from .cloud.client import CloudClient  # noqa: E402

            self._cloud = CloudClient(client_wrapper=self._client_wrapper)
        return self._cloud

    @property
    def customer(self):
        if self._customer is None:
            from .customer.client import CustomerClient  # noqa: E402

            self._customer = CustomerClient(client_wrapper=self._client_wrapper)
        return self._customer

    @property
    def export(self):
        if self._export is None:
            from .export.client import ExportClient  # noqa: E402

            self._export = ExportClient(client_wrapper=self._client_wrapper)
        return self._export

    @property
    def hosted_payment_pages(self):
        if self._hosted_payment_pages is None:
            from .hosted_payment_pages.client import HostedPaymentPagesClient  # noqa: E402

            self._hosted_payment_pages = HostedPaymentPagesClient(client_wrapper=self._client_wrapper)
        return self._hosted_payment_pages

    @property
    def import_(self):
        if self._import_ is None:
            from .import_.client import ImportClient  # noqa: E402

            self._import_ = ImportClient(client_wrapper=self._client_wrapper)
        return self._import_

    @property
    def invoice(self):
        if self._invoice is None:
            from .invoice.client import InvoiceClient  # noqa: E402

            self._invoice = InvoiceClient(client_wrapper=self._client_wrapper)
        return self._invoice

    @property
    def line_item(self):
        if self._line_item is None:
            from .line_item.client import LineItemClient  # noqa: E402

            self._line_item = LineItemClient(client_wrapper=self._client_wrapper)
        return self._line_item

    @property
    def money_in(self):
        if self._money_in is None:
            from .money_in.client import MoneyInClient  # noqa: E402

            self._money_in = MoneyInClient(client_wrapper=self._client_wrapper)
        return self._money_in

    @property
    def money_out(self):
        if self._money_out is None:
            from .money_out.client import MoneyOutClient  # noqa: E402

            self._money_out = MoneyOutClient(client_wrapper=self._client_wrapper)
        return self._money_out

    @property
    def notification(self):
        if self._notification is None:
            from .notification.client import NotificationClient  # noqa: E402

            self._notification = NotificationClient(client_wrapper=self._client_wrapper)
        return self._notification

    @property
    def notificationlogs(self):
        if self._notificationlogs is None:
            from .notificationlogs.client import NotificationlogsClient  # noqa: E402

            self._notificationlogs = NotificationlogsClient(client_wrapper=self._client_wrapper)
        return self._notificationlogs

    @property
    def ocr(self):
        if self._ocr is None:
            from .ocr.client import OcrClient  # noqa: E402

            self._ocr = OcrClient(client_wrapper=self._client_wrapper)
        return self._ocr

    @property
    def organization(self):
        if self._organization is None:
            from .organization.client import OrganizationClient  # noqa: E402

            self._organization = OrganizationClient(client_wrapper=self._client_wrapper)
        return self._organization

    @property
    def payment_link(self):
        if self._payment_link is None:
            from .payment_link.client import PaymentLinkClient  # noqa: E402

            self._payment_link = PaymentLinkClient(client_wrapper=self._client_wrapper)
        return self._payment_link

    @property
    def payment_method_domain(self):
        if self._payment_method_domain is None:
            from .payment_method_domain.client import PaymentMethodDomainClient  # noqa: E402

            self._payment_method_domain = PaymentMethodDomainClient(client_wrapper=self._client_wrapper)
        return self._payment_method_domain

    @property
    def paypoint(self):
        if self._paypoint is None:
            from .paypoint.client import PaypointClient  # noqa: E402

            self._paypoint = PaypointClient(client_wrapper=self._client_wrapper)
        return self._paypoint

    @property
    def query(self):
        if self._query is None:
            from .query.client import QueryClient  # noqa: E402

            self._query = QueryClient(client_wrapper=self._client_wrapper)
        return self._query

    @property
    def statistic(self):
        if self._statistic is None:
            from .statistic.client import StatisticClient  # noqa: E402

            self._statistic = StatisticClient(client_wrapper=self._client_wrapper)
        return self._statistic

    @property
    def subscription(self):
        if self._subscription is None:
            from .subscription.client import SubscriptionClient  # noqa: E402

            self._subscription = SubscriptionClient(client_wrapper=self._client_wrapper)
        return self._subscription

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import TemplatesClient  # noqa: E402

            self._templates = TemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates

    @property
    def token_storage(self):
        if self._token_storage is None:
            from .token_storage.client import TokenStorageClient  # noqa: E402

            self._token_storage = TokenStorageClient(client_wrapper=self._client_wrapper)
        return self._token_storage

    @property
    def user(self):
        if self._user is None:
            from .user.client import UserClient  # noqa: E402

            self._user = UserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def vendor(self):
        if self._vendor is None:
            from .vendor.client import VendorClient  # noqa: E402

            self._vendor = VendorClient(client_wrapper=self._client_wrapper)
        return self._vendor

    @property
    def wallet(self):
        if self._wallet is None:
            from .wallet.client import WalletClient  # noqa: E402

            self._wallet = WalletClient(client_wrapper=self._client_wrapper)
        return self._wallet


class Asyncpayabli:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : payabliEnvironment
        The environment to use for requests from the client. from .environment import payabliEnvironment



        Defaults to payabliEnvironment.SANDBOX



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from payabli import Asyncpayabli

    client = Asyncpayabli(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: payabliEnvironment = payabliEnvironment.SANDBOX,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._bill: typing.Optional[AsyncBillClient] = None
        self._boarding: typing.Optional[AsyncBoardingClient] = None
        self._charge_backs: typing.Optional[AsyncChargeBacksClient] = None
        self._check_capture: typing.Optional[AsyncCheckCaptureClient] = None
        self._cloud: typing.Optional[AsyncCloudClient] = None
        self._customer: typing.Optional[AsyncCustomerClient] = None
        self._export: typing.Optional[AsyncExportClient] = None
        self._hosted_payment_pages: typing.Optional[AsyncHostedPaymentPagesClient] = None
        self._import_: typing.Optional[AsyncImportClient] = None
        self._invoice: typing.Optional[AsyncInvoiceClient] = None
        self._line_item: typing.Optional[AsyncLineItemClient] = None
        self._money_in: typing.Optional[AsyncMoneyInClient] = None
        self._money_out: typing.Optional[AsyncMoneyOutClient] = None
        self._notification: typing.Optional[AsyncNotificationClient] = None
        self._notificationlogs: typing.Optional[AsyncNotificationlogsClient] = None
        self._ocr: typing.Optional[AsyncOcrClient] = None
        self._organization: typing.Optional[AsyncOrganizationClient] = None
        self._payment_link: typing.Optional[AsyncPaymentLinkClient] = None
        self._payment_method_domain: typing.Optional[AsyncPaymentMethodDomainClient] = None
        self._paypoint: typing.Optional[AsyncPaypointClient] = None
        self._query: typing.Optional[AsyncQueryClient] = None
        self._statistic: typing.Optional[AsyncStatisticClient] = None
        self._subscription: typing.Optional[AsyncSubscriptionClient] = None
        self._templates: typing.Optional[AsyncTemplatesClient] = None
        self._token_storage: typing.Optional[AsyncTokenStorageClient] = None
        self._user: typing.Optional[AsyncUserClient] = None
        self._vendor: typing.Optional[AsyncVendorClient] = None
        self._wallet: typing.Optional[AsyncWalletClient] = None

    @property
    def bill(self):
        if self._bill is None:
            from .bill.client import AsyncBillClient  # noqa: E402

            self._bill = AsyncBillClient(client_wrapper=self._client_wrapper)
        return self._bill

    @property
    def boarding(self):
        if self._boarding is None:
            from .boarding.client import AsyncBoardingClient  # noqa: E402

            self._boarding = AsyncBoardingClient(client_wrapper=self._client_wrapper)
        return self._boarding

    @property
    def charge_backs(self):
        if self._charge_backs is None:
            from .charge_backs.client import AsyncChargeBacksClient  # noqa: E402

            self._charge_backs = AsyncChargeBacksClient(client_wrapper=self._client_wrapper)
        return self._charge_backs

    @property
    def check_capture(self):
        if self._check_capture is None:
            from .check_capture.client import AsyncCheckCaptureClient  # noqa: E402

            self._check_capture = AsyncCheckCaptureClient(client_wrapper=self._client_wrapper)
        return self._check_capture

    @property
    def cloud(self):
        if self._cloud is None:
            from .cloud.client import AsyncCloudClient  # noqa: E402

            self._cloud = AsyncCloudClient(client_wrapper=self._client_wrapper)
        return self._cloud

    @property
    def customer(self):
        if self._customer is None:
            from .customer.client import AsyncCustomerClient  # noqa: E402

            self._customer = AsyncCustomerClient(client_wrapper=self._client_wrapper)
        return self._customer

    @property
    def export(self):
        if self._export is None:
            from .export.client import AsyncExportClient  # noqa: E402

            self._export = AsyncExportClient(client_wrapper=self._client_wrapper)
        return self._export

    @property
    def hosted_payment_pages(self):
        if self._hosted_payment_pages is None:
            from .hosted_payment_pages.client import AsyncHostedPaymentPagesClient  # noqa: E402

            self._hosted_payment_pages = AsyncHostedPaymentPagesClient(client_wrapper=self._client_wrapper)
        return self._hosted_payment_pages

    @property
    def import_(self):
        if self._import_ is None:
            from .import_.client import AsyncImportClient  # noqa: E402

            self._import_ = AsyncImportClient(client_wrapper=self._client_wrapper)
        return self._import_

    @property
    def invoice(self):
        if self._invoice is None:
            from .invoice.client import AsyncInvoiceClient  # noqa: E402

            self._invoice = AsyncInvoiceClient(client_wrapper=self._client_wrapper)
        return self._invoice

    @property
    def line_item(self):
        if self._line_item is None:
            from .line_item.client import AsyncLineItemClient  # noqa: E402

            self._line_item = AsyncLineItemClient(client_wrapper=self._client_wrapper)
        return self._line_item

    @property
    def money_in(self):
        if self._money_in is None:
            from .money_in.client import AsyncMoneyInClient  # noqa: E402

            self._money_in = AsyncMoneyInClient(client_wrapper=self._client_wrapper)
        return self._money_in

    @property
    def money_out(self):
        if self._money_out is None:
            from .money_out.client import AsyncMoneyOutClient  # noqa: E402

            self._money_out = AsyncMoneyOutClient(client_wrapper=self._client_wrapper)
        return self._money_out

    @property
    def notification(self):
        if self._notification is None:
            from .notification.client import AsyncNotificationClient  # noqa: E402

            self._notification = AsyncNotificationClient(client_wrapper=self._client_wrapper)
        return self._notification

    @property
    def notificationlogs(self):
        if self._notificationlogs is None:
            from .notificationlogs.client import AsyncNotificationlogsClient  # noqa: E402

            self._notificationlogs = AsyncNotificationlogsClient(client_wrapper=self._client_wrapper)
        return self._notificationlogs

    @property
    def ocr(self):
        if self._ocr is None:
            from .ocr.client import AsyncOcrClient  # noqa: E402

            self._ocr = AsyncOcrClient(client_wrapper=self._client_wrapper)
        return self._ocr

    @property
    def organization(self):
        if self._organization is None:
            from .organization.client import AsyncOrganizationClient  # noqa: E402

            self._organization = AsyncOrganizationClient(client_wrapper=self._client_wrapper)
        return self._organization

    @property
    def payment_link(self):
        if self._payment_link is None:
            from .payment_link.client import AsyncPaymentLinkClient  # noqa: E402

            self._payment_link = AsyncPaymentLinkClient(client_wrapper=self._client_wrapper)
        return self._payment_link

    @property
    def payment_method_domain(self):
        if self._payment_method_domain is None:
            from .payment_method_domain.client import AsyncPaymentMethodDomainClient  # noqa: E402

            self._payment_method_domain = AsyncPaymentMethodDomainClient(client_wrapper=self._client_wrapper)
        return self._payment_method_domain

    @property
    def paypoint(self):
        if self._paypoint is None:
            from .paypoint.client import AsyncPaypointClient  # noqa: E402

            self._paypoint = AsyncPaypointClient(client_wrapper=self._client_wrapper)
        return self._paypoint

    @property
    def query(self):
        if self._query is None:
            from .query.client import AsyncQueryClient  # noqa: E402

            self._query = AsyncQueryClient(client_wrapper=self._client_wrapper)
        return self._query

    @property
    def statistic(self):
        if self._statistic is None:
            from .statistic.client import AsyncStatisticClient  # noqa: E402

            self._statistic = AsyncStatisticClient(client_wrapper=self._client_wrapper)
        return self._statistic

    @property
    def subscription(self):
        if self._subscription is None:
            from .subscription.client import AsyncSubscriptionClient  # noqa: E402

            self._subscription = AsyncSubscriptionClient(client_wrapper=self._client_wrapper)
        return self._subscription

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import AsyncTemplatesClient  # noqa: E402

            self._templates = AsyncTemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates

    @property
    def token_storage(self):
        if self._token_storage is None:
            from .token_storage.client import AsyncTokenStorageClient  # noqa: E402

            self._token_storage = AsyncTokenStorageClient(client_wrapper=self._client_wrapper)
        return self._token_storage

    @property
    def user(self):
        if self._user is None:
            from .user.client import AsyncUserClient  # noqa: E402

            self._user = AsyncUserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def vendor(self):
        if self._vendor is None:
            from .vendor.client import AsyncVendorClient  # noqa: E402

            self._vendor = AsyncVendorClient(client_wrapper=self._client_wrapper)
        return self._vendor

    @property
    def wallet(self):
        if self._wallet is None:
            from .wallet.client import AsyncWalletClient  # noqa: E402

            self._wallet = AsyncWalletClient(client_wrapper=self._client_wrapper)
        return self._wallet


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: payabliEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PayabliCredentialsPascal(UniversalBaseModel):
    service: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Service")] = None
    mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Mode")] = None
    min_ticket: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="MinTicket")] = None
    max_ticket: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="MaxTicket")] = None
    cfee_fix: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CfeeFix")] = None
    cfee_float: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CfeeFloat")] = None
    cfee_min: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CfeeMin")] = None
    cfee_max: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CfeeMax")] = None
    account_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="AccountId")] = None
    reference_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ReferenceId")] = None
    accept_same_day_ach: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptSameDayACH")] = (
        None
    )
    currency: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Currency")] = pydantic.Field(
        default=None
    )
    """
    The default currency for the paypoint, either `USD` or `CAD`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class CardSetup(UniversalBaseModel):
    accept_amex: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptAmex")] = pydantic.Field(
        default=None
    )
    """
    Determines whether American Express is accepted.
    """

    accept_discover: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptDiscover")] = (
        pydantic.Field(default=None)
    )
    """
    Determines whether Discover is accepted.
    """

    accept_mastercard: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptMastercard")] = (
        pydantic.Field(default=None)
    )
    """
    Determines whether Mastercard is accepted.
    """

    accept_visa: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptVisa")] = pydantic.Field(
        default=None
    )
    """
    Determines whether Visa is accepted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.batch_number import BatchNumber
from ...types.dbaname import Dbaname
from ...types.deposit_date import DepositDate
from ...types.entrypointfield import Entrypointfield
from ...types.expected_deposit_date import ExpectedDepositDate
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.general_events import GeneralEvents
from ...types.org_parent_name import OrgParentName
from ...types.paypoint_id import PaypointId
from ...types.paypoint_name import PaypointName
from ...types.transfer_identifier import TransferIdentifier
from .query_batches_transfer import QueryBatchesTransfer


class QueryBatchesResponseRecordsItem(UniversalBaseModel):
    id_batch: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="IdBatch")] = pydantic.Field(
        default=None
    )
    """
    The batch ID.
    """

    batch_number: typing_extensions.Annotated[typing.Optional[BatchNumber], FieldMetadata(alias="BatchNumber")] = None
    transfer_identifier: typing_extensions.Annotated[
        typing.Optional[TransferIdentifier], FieldMetadata(alias="TransferIdentifier")
    ] = None
    events_data: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]], FieldMetadata(alias="EventsData")
    ] = pydantic.Field(default=None)
    """
    Events associated with the batch.
    """

    connector_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ConnectorName")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    batch_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="BatchDate")] = (
        pydantic.Field(default=None)
    )
    """
    The batch date.
    """

    batch_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The amount of the batch.
    """

    batch_fees_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchFeesAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The total of fees in the batch.
    """

    batch_auth_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAuthAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    batch_released_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchReleasedAmount")
    ] = pydantic.Field(default=None)
    """
    Previously held funds that have been released after a risk review.
    """

    batch_hold_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchHoldAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The total amount of the batch that's being held for fraud or risk concerns.
    """

    batch_returned_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchReturnedAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of ACH returns deducted from batch.
    """

    batch_refund_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchRefundAmount")
    ] = pydantic.Field(default=None)
    """
    The total amount of refunds deducted from batch.
    """

    batch_split_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchSplitAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total of split transactions that included split funding instructions at the time of authorization.
    """

    batch_status: typing_extensions.Annotated[int, FieldMetadata(alias="BatchStatus")] = pydantic.Field()
    """
    The batch status. See [Batch Status](/developers/references/money-in-statuses#batch-status) for more.
    """

    batch_records: typing_extensions.Annotated[int, FieldMetadata(alias="BatchRecords")] = pydantic.Field()
    """
    The number of records in the batch.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[PaypointId], FieldMetadata(alias="PaypointId")] = None
    paypoint_name: typing_extensions.Annotated[typing.Optional[PaypointName], FieldMetadata(alias="PaypointName")] = (
        None
    )
    paypoint_dba: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDba")] = None
    parent_org_name: typing_extensions.Annotated[OrgParentName, FieldMetadata(alias="ParentOrgName")] = pydantic.Field()
    """
    The entrypoint's parent org.
    """

    parent_org_id: typing_extensions.Annotated[int, FieldMetadata(alias="ParentOrgId")] = pydantic.Field()
    """
    The parent organization ID.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    entry_name: typing_extensions.Annotated[Entrypointfield, FieldMetadata(alias="EntryName")]
    bank_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BankName")] = pydantic.Field(
        default=None
    )
    """
    The bank name.
    """

    batch_type: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="BatchType")] = pydantic.Field(
        default=None
    )
    """
    The batch type.
    """

    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    The payment method used.
    """

    expected_deposit_date: typing_extensions.Annotated[
        typing.Optional[ExpectedDepositDate], FieldMetadata(alias="ExpectedDepositDate")
    ] = None
    deposit_date: typing_extensions.Annotated[typing.Optional[DepositDate], FieldMetadata(alias="DepositDate")] = None
    transfer_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="TransferDate")] = (
        pydantic.Field(default=None)
    )
    """
    The batch transfer date.
    """

    transfer: typing_extensions.Annotated[typing.Optional[QueryBatchesTransfer], FieldMetadata(alias="Transfer")] = (
        pydantic.Field(default=None)
    )
    """
    Transfer details for the batch.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .transfer_message_properties import TransferMessageProperties


class TransferMessage(UniversalBaseModel):
    id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Id")] = None
    room_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="RoomId")] = None
    user_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="UserId")] = None
    user_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="UserName")] = None
    content: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Content")] = None
    created_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="CreatedAt")] = None
    message_type: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="MessageType")] = None
    message_properties: typing_extensions.Annotated[
        typing.Optional[TransferMessageProperties], FieldMetadata(alias="MessageProperties")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
